﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Newton_Raphson
{
    class Extract
    {
        public static List<double> ExtractCoeff(string input)
        {
            double coeff = 0;
            int n = 0;
            List<double> Coeff = new List<double>();

            do
            {
                n = input.IndexOf(",");
                coeff = Convert.ToDouble(input.Substring(0, n));
                Coeff.Add(coeff);
                input = input.Substring(n + 1);
            } while (!(input.IndexOf(",") == -1));
            Coeff.Add(double.Parse(input));
            return (Coeff);
        }

    } // End ExtractCoeff Class


    public static class Complex
    {
       private static List<Double> raw = new List<Double>();

public static List<double> Subtract(double x, double jx, double y, double jy)
        {
            raw.Clear();
            raw.Add(x - y);
            raw.Add(jx - jy);
            return (raw);
        }

public static List<double> Mult(double x, double jx, double y, double jy)
        {
            raw.Clear();
            raw.Add(x * y - jx * jy);
            raw.Add(x * jy + jx * y);
            return (raw);
        }

public static List<double> Div(double x, double jx, double y, double jy)
{
    if (y != 0 && jy != 0)
    {
        raw.Clear();
        double denominator = Math.Pow(y, 2) + Math.Pow(jy, 2);
        Mult(x, jx, y, -jy);
        raw.Add(raw[0] / denominator);
        raw.Add(raw[1] / denominator);
        raw.RemoveRange(0, 2);
        return (raw);
    }
    else if (jy == 0)
    {
        raw.Clear();
        raw.Add(x / y);
        raw.Add(jx / y);
        return (raw);
    }

    else
    {
        raw.Clear();
        raw.Add((jx * jy) / Math.Pow(jy, 2));
        raw.Add((-1 * x * jy) / Math.Pow(jy, 2));
        return (raw);
    }
}
 public static List<double> Power(double x, double jx, double power)
        {
            raw.Clear();
            
            if (jx != 0)
            {
                double modulus = Math.Sqrt(Math.Pow(x, 2) + Math.Pow(jx, 2));
                double angle = 0;

                if (x < 0 && jx > 0)
                { angle = Math.PI - Math.Atan(jx / Math.Abs(x)); }
                else if (x > 0 && jx < 0)
                { angle = -Math.Atan(Math.Abs(jx) / x); }
                else if (x < 0 && jx < 0)
                { angle = (Math.PI) + Math.Atan((jx) / x); }
                else
                { angle = Math.Atan(jx / x); }

                raw.Add(Math.Pow(modulus, power) * Math.Cos(power * angle));
                raw.Add(Math.Pow(modulus, power) * Math.Sin(power * angle));
            }
            else
            {
                raw.Add(Math.Pow(x, power));
                raw.Add(0);
            }

            return (raw);
        }
    } // End Complex Class


    public static class SubstDiff
    {
        public static List<double> SubDiff(List<double> coeff, List<double> jcoeff, double real, double imaginary, int derivative)
        {
            List<double> raw = new List<double>();
            int degree = coeff.Count - 1; int n = 0; int s = 0;
            double fx = 0; double jfx = 0;
            double diffmult = 1;




            for (n = 1; n <= coeff.Count - derivative - 1; n++)
            {
                if (derivative != 0) // This will find the number to multiply when differentiated
                {
                    do
                    {
                        diffmult = (degree - s) * diffmult;
                        s++;
                    } while (!(s == derivative));
                }



                raw = Complex.Power(real, imaginary, (degree - derivative));

                raw = Complex.Mult(coeff[n - 1], jcoeff[n - 1], raw[0], raw[1]);

                fx += diffmult * raw[0];
                jfx += diffmult * raw[1];

                degree -= 1; // Reduce degree by 1
                diffmult = 1; // Return it back to default
                s = 0;

            }

            fx += coeff[n - 1];
            jfx += jcoeff[n - 1];

            raw.Clear();
            raw.Add(fx);
            raw.Add(jfx);
            return (raw);
        }

    }



    public static class Deflate
    {
        public static void Horner(ref List<double> coeff, ref List<double> jcoeff, double x, double jx)
        {
            int z = coeff.Count;
            jcoeff.Add(jcoeff[0]);
            coeff.Add(coeff[0]);
            double a = 0;
            double b = 0;
            int c = coeff.Count;

            for (int n = 1; n <= c - 3; n++)
            {
                List<double> raw = Complex.Mult(x, jx, coeff[coeff.Count - 1], jcoeff[coeff.Count - 1]);

                a = coeff[n] + raw[0];
                b = jcoeff[n] + raw[1];
                coeff.Add(a);
                jcoeff.Add(b);
            }
            coeff.RemoveRange(0, z);
            jcoeff.RemoveRange(0, z);
        }

    } // End Deflate Class





}
